@@ -1,5 +1,6 @@
 /*
  Copyright (C) 2009 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2009 Alexander Butenko <a.butenka@gmail.com>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -19,6 +20,43 @@
     #include <unistd.h>
 #endif
 
+static gchar *
+adblock_fixup_regexp (gchar* src)
+{
+    gchar* dst;
+    gchar* s;
+    /* FIXME: Avoid always allocating twice the string */
+    s = dst = g_malloc (strlen (src) * 2);
+
+    while (*src)
+    {
+        switch (*src)
+        {
+        case '*':
+            *s++ = '.';
+            break;
+        case '.':
+            *s++ = '\\';
+            break;
+        case '?':
+            *s++ = '\\';
+            break;
+        case '|':
+            *s++ = '\\';
+            break;
+        }
+        *s++ = *src;
+        src++;
+    }
+    *s = 0;
+    return dst;
+}
+
+static void
+adblock_browser_populate_tool_menu_cb (MidoriBrowser*   browser,
+                                       GtkWidget*       menu,
+                                       MidoriExtension* extension);
+
 static void
 adblock_app_add_browser_cb (MidoriApp*       app,
                             MidoriBrowser*   browser,
@@ -26,13 +64,14 @@ adblock_app_add_browser_cb (MidoriApp*       app,
 
 static void
 adblock_deactivate_cb (MidoriExtension* extension,
-                       GtkWidget*       menuitem)
+                       MidoriBrowser*   browser)
 {
     MidoriApp* app = midori_extension_get_app (extension);
 
-    gtk_widget_destroy (menuitem);
     g_signal_handlers_disconnect_by_func (
-        extension, adblock_deactivate_cb, menuitem);
+        browser, adblock_browser_populate_tool_menu_cb, extension);
+    g_signal_handlers_disconnect_by_func (
+        extension, adblock_deactivate_cb, browser);
     g_signal_handlers_disconnect_by_func (
         app, adblock_app_add_browser_cb, extension);
     /* FIXME: Disconnect session callbacks */
@@ -256,26 +295,28 @@ adblock_menu_configure_filters_activate_cb (GtkWidget*       menuitem,
 }
 
 static void
-adblock_app_add_browser_cb (MidoriApp*       app,
-                            MidoriBrowser*   browser,
-                            MidoriExtension* extension)
+adblock_browser_populate_tool_menu_cb (MidoriBrowser*   browser,
+                                       GtkWidget*       menu,
+                                       MidoriExtension* extension)
 {
-    GtkWidget* panel;
-    GtkWidget* menu;
     GtkWidget* menuitem;
 
-    panel = katze_object_get_object (browser, "panel");
-    menu = katze_object_get_object (panel, "menu");
     menuitem = gtk_menu_item_new_with_mnemonic (_("Configure _Advertisement filters..."));
     g_signal_connect (menuitem, "activate",
         G_CALLBACK (adblock_menu_configure_filters_activate_cb), extension);
     gtk_widget_show (menuitem);
-    gtk_menu_shell_insert (GTK_MENU_SHELL (menu), menuitem, 3);
-    g_object_unref (menu);
-    g_object_unref (panel);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+}
 
+static void
+adblock_app_add_browser_cb (MidoriApp*       app,
+                            MidoriBrowser*   browser,
+                            MidoriExtension* extension)
+{
+    g_signal_connect (browser, "populate-tool-menu",
+        G_CALLBACK (adblock_browser_populate_tool_menu_cb), extension);
     g_signal_connect (extension, "deactivate",
-        G_CALLBACK (adblock_deactivate_cb), menuitem);
+        G_CALLBACK (adblock_deactivate_cb), browser);
 }
 
 static gboolean
@@ -322,12 +363,7 @@ adblock_parse_line (gchar* line)
     if (line[0] == '[')
         return NULL;
     g_strchomp (line);
-    /* TODO: Replace trailing '*' with '.*' */
-    if (line[0] == '*')
-        return g_strconcat (".", line, NULL);
-    else if (line[0] == '?')
-        return g_strconcat ("\\", line, NULL);
-    return g_strdup (line);
+    return adblock_fixup_regexp (line);
 }
 
 static GHashTable*
@@ -469,11 +505,11 @@ test_adblock_parse (void)
 
     g_assert_cmpstr (adblock_parse_line ("*foo"), ==, ".*foo");
     g_assert_cmpstr (adblock_parse_line ("?foo"), ==, "\\?foo");
-    /* g_assert_cmpstr (adblock_parse_line ("foo*"), ==, "foo.*");
-    g_assert_cmpstr (adblock_parse_line ("foo?"), ==, "foo\\?"); */
+    g_assert_cmpstr (adblock_parse_line ("foo*"), ==, "foo.*");
+    g_assert_cmpstr (adblock_parse_line ("foo?"), ==, "foo\\?");
 
-    g_assert_cmpstr (adblock_parse_line (".*foo/bar"), ==, ".*foo/bar");
-    g_assert_cmpstr (adblock_parse_line ("http://bla.blub/.*"), ==, "http://bla.blub/.*");
+    g_assert_cmpstr (adblock_parse_line (".*foo/bar"), ==, "\\..*foo/bar");
+    g_assert_cmpstr (adblock_parse_line ("http://bla.blub/*"), ==, "http://bla\\.blub/.*");
 }
 
 static void
@@ -486,10 +522,10 @@ test_adblock_pattern (void)
     temp = g_file_open_tmp ("midori_adblock_match_test_XXXXXX", &filename, NULL);
 
     g_file_set_contents (filename,
-        "*ads.foo.bar.*\n"
-        ".*ads.bogus.name.*\n"
-        "http://ads.bla.blub/.*\n"
-        "http://ads.blub.boing/*.",
+        "*ads.foo.bar*\n"
+        "*ads.bogus.name*\n"
+        "http://ads.bla.blub/*\n"
+        "http://ads.blub.boing/*",
         -1, NULL);
     pattern = adblock_parse_file (filename);
 
