@@ -23,6 +23,7 @@ struct _MidoriPanel
 {
     GtkHBox parent_instance;
 
+    GtkWidget* labelbar;
     GtkWidget* toolbar;
     GtkToolItem* button_align;
     GtkToolItem* button_detach;
@@ -32,6 +33,8 @@ struct _MidoriPanel
     GtkWidget* notebook;
     GtkMenu*   menu;
 
+    gboolean show_titles;
+    gboolean show_controls;
     gboolean right_aligned;
 };
 
@@ -53,6 +56,8 @@ enum
     PROP_SHADOW_TYPE,
     PROP_MENU,
     PROP_PAGE,
+    PROP_SHOW_TITLES,
+    PROP_SHOW_CONTROLS,
     PROP_RIGHT_ALIGNED,
 };
 
@@ -133,6 +138,15 @@ midori_panel_class_init (MidoriPanelClass* class)
                                      GTK_SHADOW_NONE,
                                      flags));
 
+    /**
+     * MidoriWebSettings:menu:
+     *
+     * This is the menu that holds the panel menu items.
+     *
+     * You shouldn't use this menu or add items.
+     *
+     * Deprecated: 0.1.9
+     */
     g_object_class_install_property (gobject_class,
                                      PROP_MENU,
                                      g_param_spec_object (
@@ -151,6 +165,38 @@ midori_panel_class_init (MidoriPanelClass* class)
                                      -1, G_MAXINT, -1,
                                      flags));
 
+    /**
+     * MidoriWebSettings:show-titles:
+     *
+     * Whether to show panel titles.
+     *
+     * Since: 0.1.9
+     */
+    g_object_class_install_property (gobject_class,
+                                     PROP_SHOW_TITLES,
+                                     g_param_spec_boolean (
+                                     "show-titles",
+                                     "Show Titles",
+                                     "Whether to show panel titles",
+                                     TRUE,
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+    /**
+     * MidoriWebSettings:show-controls:
+     *
+     * Whether to show operating controls.
+     *
+     * Since: 0.1.9
+     */
+    g_object_class_install_property (gobject_class,
+                                     PROP_SHOW_CONTROLS,
+                                     g_param_spec_boolean (
+                                     "show-controls",
+                                     "Show Controls",
+                                     "Whether to show operating controls",
+                                     TRUE,
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
     /**
     * MidoriWebSettings:right-aligned:
     *
@@ -301,6 +347,8 @@ midori_panel_init (MidoriPanel* panel)
     GtkWidget* labelbar;
     GtkToolItem* toolitem;
 
+    panel->show_titles = TRUE;
+    panel->show_controls = TRUE;
     panel->right_aligned = FALSE;
 
     /* Create the sidebar */
@@ -317,6 +365,7 @@ midori_panel_init (MidoriPanel* panel)
 
     /* Create the titlebar */
     labelbar = gtk_toolbar_new ();
+    panel->labelbar = labelbar;
     gtk_toolbar_set_icon_size (GTK_TOOLBAR (labelbar), GTK_ICON_SIZE_MENU);
     gtk_toolbar_set_style (GTK_TOOLBAR (labelbar), GTK_TOOLBAR_ICONS);
     toolitem = gtk_tool_item_new ();
@@ -329,6 +378,8 @@ midori_panel_init (MidoriPanel* panel)
     toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_FULLSCREEN);
     gtk_widget_set_sensitive (GTK_WIDGET (toolitem), FALSE);
     panel->button_detach = toolitem;
+    gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem),
+        _("Detach chosen panel from the window"));
     gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (toolitem),
         _("Detach chosen panel from the window"));
     g_signal_connect (toolitem, "clicked",
@@ -339,6 +390,8 @@ midori_panel_init (MidoriPanel* panel)
     gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, -1);
     #endif
     toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD);
+    gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem),
+        _("Align sidepanel to the right"));
     gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (toolitem),
         _("Align sidepanel to the right"));
     g_signal_connect (toolitem, "clicked",
@@ -404,11 +457,23 @@ midori_panel_set_property (GObject*      object,
         break;
     case PROP_MENU:
         katze_object_assign (panel->menu, g_value_dup_object (value));
-        /* FIXME: Move existing items to the new menu */
         break;
     case PROP_PAGE:
         midori_panel_set_current_page (panel, g_value_get_int (value));
         break;
+    case PROP_SHOW_TITLES:
+        panel->show_titles = g_value_get_boolean (value);
+        #if HAVE_HILDON
+        panel->show_titles = TRUE;
+        #endif
+        gtk_toolbar_set_style (GTK_TOOLBAR (panel->toolbar),
+            panel->show_titles ? GTK_TOOLBAR_BOTH : GTK_TOOLBAR_ICONS);
+        break;
+    case PROP_SHOW_CONTROLS:
+        panel->show_controls = g_value_get_boolean (value);
+        sokoke_widget_set_visible (panel->labelbar, panel->show_controls);
+        sokoke_widget_set_visible (panel->toolbar, panel->show_controls);
+        break;
     case PROP_RIGHT_ALIGNED:
         midori_panel_set_right_aligned (panel, g_value_get_boolean (value));
         break;
@@ -438,6 +503,12 @@ midori_panel_get_property (GObject*    object,
     case PROP_PAGE:
         g_value_set_int (value, midori_panel_get_current_page (panel));
         break;
+    case PROP_SHOW_TITLES:
+        g_value_set_boolean (value, panel->show_titles);
+        break;
+    case PROP_SHOW_CONTROLS:
+        g_value_set_boolean (value, panel->show_controls);
+        break;
     case PROP_RIGHT_ALIGNED:
         g_value_set_boolean (value, panel->right_aligned);
         break;
@@ -468,6 +539,8 @@ midori_panel_new (void)
  * @compact: %TRUE if the panel should be compact
  *
  * Determines if the panel should be compact.
+ *
+ * Deprecated: 0.1.9
  **/
 void
 midori_panel_set_compact (MidoriPanel* panel,
@@ -475,11 +548,7 @@ midori_panel_set_compact (MidoriPanel* panel,
 {
     g_return_if_fail (MIDORI_IS_PANEL (panel));
 
-    #if HAVE_HILDON
-    compact = TRUE;
-    #endif
-    gtk_toolbar_set_style (GTK_TOOLBAR (panel->toolbar),
-        compact ? GTK_TOOLBAR_ICONS : GTK_TOOLBAR_BOTH);
+    g_object_set (panel, "show-titles", !compact, NULL);
 }
 
 /**
@@ -504,6 +573,9 @@ midori_panel_set_right_aligned (MidoriPanel* panel,
     gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (panel->button_align),
         right_aligned ? GTK_STOCK_GO_BACK : GTK_STOCK_GO_FORWARD);
     panel->right_aligned = right_aligned;
+    gtk_tool_button_set_label (GTK_TOOL_BUTTON (panel->button_align),
+        !panel->right_aligned ? _("Align sidepanel to the right")
+            : _("Align sidepanel to the left"));
     gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (panel->button_align),
         !panel->right_aligned ? _("Align sidepanel to the right")
             : _("Align sidepanel to the left"));
@@ -515,26 +587,46 @@ midori_panel_menu_item_activate_cb (GtkWidget*   widget,
                                     MidoriPanel* panel)
 {
     GtkWidget* child;
-    GtkToggleToolButton* toolitem;
+    GtkToolItem* toolitem;
     guint n;
 
     child = g_object_get_data (G_OBJECT (widget), "page");
-    toolitem = g_object_get_data (G_OBJECT (widget), "toolitem");
+    n = midori_panel_page_num (panel, child);
+    toolitem = gtk_toolbar_get_nth_item (GTK_TOOLBAR (panel->toolbar), n);
 
     if (toolitem)
     {
         /* Unsetting the button before setting it ensures that
            it will emit signals even if it was active before */
-        gtk_toggle_tool_button_set_active (toolitem, FALSE);
-        gtk_toggle_tool_button_set_active (toolitem, TRUE);
-    }
-    else
-    {
-        n = midori_panel_page_num (panel, child);
-        midori_panel_set_current_page (panel, n);
-        g_signal_emit (panel, signals[SWITCH_PAGE], 0, n);
-        gtk_widget_show (GTK_WIDGET (panel));
+        GtkToggleToolButton* button = GTK_TOGGLE_TOOL_BUTTON (toolitem);
+        g_signal_handlers_block_by_func (widget,
+            midori_panel_menu_item_activate_cb, panel);
+        gtk_toggle_tool_button_set_active (button, FALSE);
+        gtk_toggle_tool_button_set_active (button, TRUE);
+        g_signal_handlers_unblock_by_func (widget,
+            midori_panel_menu_item_activate_cb, panel);
     }
+
+    midori_panel_set_current_page (panel, n);
+    g_signal_emit (panel, signals[SWITCH_PAGE], 0, n);
+    gtk_widget_show (GTK_WIDGET (panel));
+}
+
+/* Private function, used by MidoriBrowser */
+/* static */ GtkWidget*
+midori_panel_construct_menu_item (MidoriPanel*    panel,
+                                  MidoriViewable* viewable)
+{
+    const gchar* stock_id;
+    GtkWidget* menuitem;
+
+    stock_id = midori_viewable_get_stock_id (viewable);
+    menuitem = gtk_image_menu_item_new_from_stock (stock_id, NULL);
+    gtk_widget_show (menuitem);
+    g_object_set_data (G_OBJECT (menuitem), "page", viewable);
+    g_signal_connect (menuitem, "activate",
+                      G_CALLBACK (midori_panel_menu_item_activate_cb), panel);
+    return menuitem;
 }
 
 static void
@@ -582,6 +674,54 @@ midori_panel_construct_tool_item (MidoriPanel*    panel,
     return toolitem;
 }
 
+static void
+midori_panel_show_titles_toggled_cb (GtkWidget*   menuitem,
+                                     MidoriPanel* panel)
+{
+    g_object_set (panel, "show-titles", !panel->show_titles, NULL);
+}
+
+static void
+midori_panel_show_controls_toggled_cb (GtkWidget*   menuitem,
+                                       MidoriPanel* panel)
+{
+    g_object_set (panel, "show-controls", !panel->show_controls, NULL);
+}
+
+static void
+midori_panel_options_clicked_cb (GtkToolItem* toolitem,
+                                 MidoriPanel* panel)
+{
+    gint n;
+    GtkWidget* viewable;
+    GtkWidget* menu;
+    GtkWidget* menuitem;
+
+    n = midori_panel_get_current_page (panel);
+    viewable = midori_panel_get_nth_page (panel, n);
+    menu = gtk_menu_new ();
+    #if !HAVE_HILDON
+    menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show panel _titles"));
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
+                                    panel->show_titles);
+    g_signal_connect (menuitem, "toggled",
+        G_CALLBACK (midori_panel_show_titles_toggled_cb), panel);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+    gtk_widget_show (menuitem);
+    #endif
+    menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show operating _controls"));
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
+                                    panel->show_controls);
+    g_signal_connect (menuitem, "toggled",
+        G_CALLBACK (midori_panel_show_controls_toggled_cb), panel);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+    gtk_widget_show (menuitem);
+    g_signal_emit_by_name (viewable, "populate-option-menu", menu);
+
+    katze_widget_popup (GTK_WIDGET (toolitem), GTK_MENU (menu),
+                        NULL, SOKOKE_MENU_POSITION_LEFT);
+}
+
 /**
  * midori_panel_append_page:
  * @panel: a #MidoriPanel
@@ -608,10 +748,8 @@ midori_panel_append_page (MidoriPanel*    panel,
     GObjectClass* gobject_class;
     GtkWidget* widget;
     GtkWidget* toolbar;
-    const gchar* label;
-    const gchar* stock_id;
     GtkToolItem* toolitem;
-    GtkWidget* menuitem;
+    const gchar* label;
     guint n;
 
     g_return_val_if_fail (MIDORI_IS_PANEL (panel), -1);
@@ -641,6 +779,12 @@ midori_panel_append_page (MidoriPanel*    panel,
     gtk_container_add (GTK_CONTAINER (panel->notebook), scrolled);
 
     toolbar = midori_viewable_get_toolbar (viewable);
+    toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_PROPERTIES);
+    gtk_tool_item_set_tooltip_text (toolitem, _("Options"));
+    g_signal_connect (toolitem, "clicked",
+        G_CALLBACK (midori_panel_options_clicked_cb), panel);
+    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, 0);
+    gtk_widget_show (GTK_WIDGET (toolitem));
     gtk_widget_show (toolbar);
     gtk_container_add (GTK_CONTAINER (panel->toolbook), toolbar);
     g_signal_connect (viewable, "destroy",
@@ -648,26 +792,9 @@ midori_panel_append_page (MidoriPanel*    panel,
 
     n = midori_panel_page_num (panel, scrolled);
     label = midori_viewable_get_label (viewable);
-    stock_id = midori_viewable_get_stock_id (viewable);
-
-    toolitem = midori_panel_construct_tool_item (panel, viewable);
-
-    if (panel->menu)
-    {
-        menuitem = gtk_image_menu_item_new_from_stock (stock_id, NULL);
-        gtk_widget_show (menuitem);
-        g_object_set_data (G_OBJECT (menuitem), "page", viewable);
-        g_object_set_data (G_OBJECT (menuitem), "toolitem", toolitem);
-        g_signal_connect (menuitem, "activate",
-                          G_CALLBACK (midori_panel_menu_item_activate_cb),
-                          panel);
-        gtk_menu_shell_insert (GTK_MENU_SHELL (panel->menu), menuitem, 4);
-        g_object_set_data (G_OBJECT (scrolled), "panel-menuitem", menuitem);
-        g_signal_connect (viewable, "destroy",
-                          G_CALLBACK (midori_panel_widget_destroy_cb), menuitem);
-    }
 
     g_object_set_data (G_OBJECT (viewable), "parent", scrolled);
+    midori_panel_construct_tool_item (panel, viewable);
     g_signal_connect (viewable, "destroy",
                       G_CALLBACK (midori_panel_viewable_destroy_cb), panel);
 
